/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import ch.hortis.sonar.mvn.reports.CPDHandler;
import ch.hortis.sonar.mvn.reports.CloverHandler;
import ch.hortis.sonar.mvn.reports.CoberturaHandler;
import ch.hortis.sonar.mvn.reports.ReportHandler;
import ch.hortis.sonar.mvn.reports.SurefireHandler;
import org.sonar.maven.clover.CloverCollector;
import org.sonar.maven.cobertura.CoberturaCollector;
import org.sonar.maven.cpd.CpdCollector;
import org.sonar.maven.surefire.SurefireCollector;
import org.sonar.plugins.maven.MavenUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public enum Report {
  /*
  IMPORTANT NOTE : 
  
  1. clover and cobertura execute surefire to check code coverage. That overrides surefire XML files. 
     Execution time is not representative (code is instrumented) so surefire must be executed AFTER cobertura/clover.
     
   */
  CPD( MavenUtils.APACHE_MOJO_GROUP_ID, "maven-pmd-plugin",
       "2.3", CpdCollector.class, new CPDHandler(), true, true ),
  Cobertura( MavenUtils.CODEHAUS_MOJO_GROUP_ID, "cobertura-maven-plugin",
             "2.2", CoberturaCollector.class, new CoberturaHandler(), false, false ),
  Clover( MavenUtils.APACHE_MOJO_GROUP_ID, "maven-clover-plugin",
          "2.4", CloverCollector.class, new CloverHandler(), false, false ),
  Surefire( MavenUtils.APACHE_MOJO_GROUP_ID, "maven-surefire-plugin",
            "2.4.3", SurefireCollector.class, new SurefireHandler(), false, false );

  private String groupId;
  private String artifactId;
  private String version;
  private boolean includeInLightReport;
  private Class collectorClass;
  private ReportHandler reportHandler;
  private boolean versionFixed;

  private Report( String groupId, String artifactId,
      String version, Class collectorClass, ReportHandler reportHandler,
      boolean includeInLightReport, boolean versionFixed ) {
    this.groupId = groupId;
    this.artifactId = artifactId;
    this.version = version;
    this.collectorClass = collectorClass;
    this.reportHandler = reportHandler;
    if (this.reportHandler!=null) {
      this.reportHandler.setReport( this );
    }
    this.includeInLightReport = includeInLightReport;
    this.versionFixed = versionFixed;
  }

  public String getGroupId() {
    return groupId;
  }

  public String getArtifactId() {
    return artifactId;
  }

  public String getVersion() {
    return version;
  }

  public boolean isIncludeInLightReport(){
    return includeInLightReport;
  }
  
  public boolean isVersionFixed(){
    return versionFixed;
  }  

  public String toString() {
    return getGroupId() + ":" + getArtifactId() + ":" + getVersion();
  }

  public Class getCollectorClass() {
    return collectorClass;
  }

  public ReportHandler getReportHandler() {
    return reportHandler;
  }
  
  private static List<Report> getReports(boolean includeInLightReport) {
    if (includeInLightReport) {
      ArrayList<Report> reports = new ArrayList<Report>();
      for (Report report : getReports()) {
        if (report.isIncludeInLightReport()) {
          reports.add(report);
        }
      }
      return reports;
    }
    return Arrays.asList(Report.values());
  }  

  public static List<Report> getReports() {
    return getReports(false);
  }

  public static List<Report> getLightReports() {
    return getReports(true);
  }
}